import { classes } from '@automapper/classes';
import { AutomapperModule } from '@automapper/nestjs';
import { Test, TestingModule } from '@nestjs/testing';
import { AuthRepository } from '../../adapters/secondaries/auth.repository';
import { Auth } from '../../domain/entities/auth';
import * as bcrypt from 'bcrypt';
import { ValidateAuthUseCase } from '../../domain/usecases/validate-auth.usecase';
import { ValidateAuthQuery } from '../../queries/validate-auth.query';
import { UsernameRepository } from '../../adapters/secondaries/username.repository';
import { Type } from '../../domain/dtos/type.enum';
import { NotFoundException, UnauthorizedException } from '@nestjs/common';
import { DatabaseException } from '../../../database/src/exceptions/DatabaseException';

const mockAuthRepository = {
  findOne: jest
    .fn()
    .mockImplementationOnce(() => ({
      uuid: 'bb281075-1b98-4456-89d6-c643d3044a91',
      password: bcrypt.hashSync('John123', 10),
    }))
    .mockImplementationOnce(() => ({
      uuid: 'bb281075-1b98-4456-89d6-c643d3044a91',
      password: bcrypt.hashSync('John123', 10),
    })),
};

const mockUsernameRepository = {
  findOne: jest
    .fn()
    .mockImplementationOnce(() => ({
      uuid: 'bb281075-1b98-4456-89d6-c643d3044a91',
      username: 'john.doe@email.com',
      type: Type.EMAIL,
    }))
    .mockImplementationOnce(() => {
      throw new DatabaseException();
    })
    .mockImplementationOnce(() => ({
      uuid: 'bb281075-1b98-4456-89d6-c643d3044a91',
      username: 'john.doe@email.com',
      type: Type.EMAIL,
    })),
};

describe('ValidateAuthUseCase', () => {
  let validateAuthUseCase: ValidateAuthUseCase;

  beforeAll(async () => {
    const module: TestingModule = await Test.createTestingModule({
      imports: [AutomapperModule.forRoot({ strategyInitializer: classes() })],
      providers: [
        {
          provide: AuthRepository,
          useValue: mockAuthRepository,
        },
        {
          provide: UsernameRepository,
          useValue: mockUsernameRepository,
        },
        ValidateAuthUseCase,
      ],
    }).compile();

    validateAuthUseCase = module.get<ValidateAuthUseCase>(ValidateAuthUseCase);
  });

  it('should be defined', () => {
    expect(validateAuthUseCase).toBeDefined();
  });

  describe('execute', () => {
    it('should validate an auth and returns entity object', async () => {
      const auth: Auth = await validateAuthUseCase.execute(
        new ValidateAuthQuery('john.doe@email.com', 'John123'),
      );

      expect(auth.uuid).toBe('bb281075-1b98-4456-89d6-c643d3044a91');
    });

    it('should not validate an auth with unknown username and returns not found exception', async () => {
      await expect(
        validateAuthUseCase.execute(
          new ValidateAuthQuery('jane.doe@email.com', 'Jane123'),
        ),
      ).rejects.toBeInstanceOf(NotFoundException);
    });

    it('should not validate an auth with wrong password and returns unauthorized exception', async () => {
      await expect(
        validateAuthUseCase.execute(
          new ValidateAuthQuery('john.doe@email.com', 'John1234'),
        ),
      ).rejects.toBeInstanceOf(UnauthorizedException);
    });
  });
});
